@@ -16,26 +16,14 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
-#ifndef __MG_PHASE_DIALOG_H__
-#define __MG_PHASE_DIALOG_H__
+#pragma once
 
-#include <gtk/gtkwidget.h>
+#include <gtk/gtk.h>
 #include <libplanner/mrp-project.h>
-#include "planner-main-window.h"
-
-GtkWidget * planner_phase_dialog_new (MgMainWindow *window);
-
-#endif /* __MG_PHASE_DIALOG_H__ */
-
-
-
-
-
-
-
-
+#include "planner-window.h"
 
+GtkWidget * planner_phase_dialog_new (PlannerWindow *window);
