@@ -16,14 +16,13 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
-#ifndef __PLANNER_TABLE_PRINT_SHEET_H__
-#define __PLANNER_TABLE_PRINT_SHEET_H__
+#pragma once
 
-#include <gtk/gtktreeview.h>
+#include <gtk/gtk.h>
 #include "planner-print-job.h"
 #include "planner-view.h"
 
@@ -36,5 +35,3 @@ PlannerTablePrintSheet *planner_table_print_sheet_new         (PlannerView
 							       PlannerPrintJob        *job,
 							       GtkTreeView            *tree);
 void                    planner_table_print_sheet_free        (PlannerTablePrintSheet *sheet);
-
-#endif /* __PLANNER_TABLE_PRINT_SHEET_H__ */
