@@ -5,6 +5,7 @@
  * Copyright (C) 2003 Xavier Ordoquy <xordoquy@wanadoo.fr>
  * Copyright (C) 2006 Alvaro del Castillo <acs@barrapunto.com>
  * Copyright (C) 2008 Lee Baylis <lee@leebaylis.co.uk>
+ * Copyright (C) 2020 Mart Raudsepp <mart@leio.tech>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,8 +19,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /* FIXME: This code needs a SERIOUS clean-up. */
@@ -39,7 +40,6 @@
 #include "planner-usage-row.h"
 #include "planner-usage-chart.h"
 #include "planner-canvas-line.h"
-#include "eel-canvas-rect.h"
 #include "planner-scale-utils.h"
 #include "planner-usage-chart.h"
 #include "planner-usage-model.h"
@@ -87,10 +87,6 @@ typedef enum {
 } State;
 
 struct _PlannerUsageRowPriv {
-        GdkGC         *complete_gc;
-        GdkGC         *break_gc;
-        GdkGC         *fill_gc;
-        GdkGC         *frame_gc;
         PangoLayout   *layout;
 
         MrpAssignment *assignment;
@@ -116,7 +112,7 @@ struct _PlannerUsageRowPriv {
 
 static void     usage_row_class_init                   (PlannerUsageRowClass  *class);
 static void     usage_row_init                         (PlannerUsageRow       *row);
-static void     usage_row_destroy                      (GtkObject              *object);
+static void     usage_row_dispose                      (GObject               *object);
 static void     usage_row_set_property                 (GObject                *object,
 							 guint                   param_id,
 							 const GValue           *value,
@@ -126,23 +122,19 @@ static void     usage_row_get_property                 (GObject                *
 							 GValue                 *value,
 							 GParamSpec             *pspec);
 static void     usage_row_update                       (GnomeCanvasItem        *item,
-							 double                 *affine,
-							 ArtSVP                 *clip_path,
-							 int                     flags);
-static void     usage_row_realize                      (GnomeCanvasItem        *item);
-static void     usage_row_unrealize                    (GnomeCanvasItem        *item);
+							const cairo_matrix_t  *i2c,
+							int                     flags);
 static void     usage_row_draw                         (GnomeCanvasItem        *item,
-							 GdkDrawable            *drawable,
+							 cairo_t               *cr,
 							 gint                    x,
 							 gint                    y,
 							 gint                    width,
 							 gint                    height);
-static double   usage_row_point                        (GnomeCanvasItem        *item,
+static GnomeCanvasItem * usage_row_point               (GnomeCanvasItem        *item,
 							 double                  x,
 							 double                  y,
 							 gint                    cx,
-							 gint                    cy,
-							 GnomeCanvasItem       **actual_item);
+							 gint                    cy);
 static void     usage_row_bounds                       (GnomeCanvasItem        *item,
 							 double                 *x1,
 							 double                 *y1,
@@ -170,26 +162,6 @@ static void     usage_row_resource_assignment_added_cb (MrpResource            *
 
 static GnomeCanvasItemClass *parent_class;
 static guint                 signals[LAST_SIGNAL];
-static GdkBitmap            *complete_stipple = NULL;
-static gchar                 complete_stipple_pattern[] = { 0x02, 0x01 };
-static GdkBitmap            *break_stipple = NULL;
-static gchar                 break_stipple_pattern[] = { 0x03 };
-
-static GdkColor              color_normal;
-static GdkColor              color_normal_light;
-static GdkColor              color_normal_dark;
-
-static GdkColor              color_free;
-static GdkColor              color_free_light;
-static GdkColor              color_free_dark;
-
-static GdkColor              color_underuse;
-static GdkColor              color_underuse_light;
-static GdkColor              color_underuse_dark;
-
-static GdkColor              color_overuse;
-static GdkColor              color_overuse_light;
-static GdkColor              color_overuse_dark;
 
 
 GType
@@ -219,11 +191,9 @@ static void
 usage_row_class_init (PlannerUsageRowClass * class)
 {
         GObjectClass         *gobject_class;
-        GtkObjectClass       *object_class;
         GnomeCanvasItemClass *item_class;
 
         gobject_class = (GObjectClass *) class;
-        object_class = (GtkObjectClass *) class;
         item_class = (GnomeCanvasItemClass *) class;
 
         parent_class = g_type_class_peek_parent (class);
@@ -283,14 +253,14 @@ usage_row_class_init (PlannerUsageRowClass * class)
                  g_param_spec_double ("height", NULL, NULL,
                                       -G_MAXDOUBLE, G_MAXDOUBLE, 0.0,
                                       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
-	
+
         g_object_class_install_property
                 (gobject_class,
                  PROP_ASSIGNMENT,
                  g_param_spec_object ("assignment", NULL, NULL,
                                       MRP_TYPE_ASSIGNMENT,
                                       G_PARAM_READWRITE));
-	
+
         g_object_class_install_property
                 (gobject_class,
                  PROP_RESOURCE,
@@ -303,11 +273,9 @@ usage_row_class_init (PlannerUsageRowClass * class)
                  g_param_spec_boolean ("highlight", NULL, NULL,
                                        FALSE, G_PARAM_READWRITE));
 
-        object_class->destroy = usage_row_destroy;
+        gobject_class->dispose = usage_row_dispose;
 
         item_class->update = usage_row_update;
-        item_class->realize = usage_row_realize;
-        item_class->unrealize = usage_row_unrealize;
         item_class->draw = usage_row_draw;
         item_class->point = usage_row_point;
         item_class->bounds = usage_row_bounds;
@@ -335,7 +303,7 @@ usage_row_init (PlannerUsageRow * row)
 }
 
 static void
-usage_row_destroy (GtkObject * object)
+usage_row_dispose (GObject *object)
 {
         PlannerUsageRow     *row;
         PlannerUsageRowPriv *priv;
@@ -355,8 +323,8 @@ usage_row_destroy (GtkObject * object)
                 row->priv = NULL;
         }
 
-        if (GTK_OBJECT_CLASS (parent_class)->destroy) {
-                (*GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+        if (G_OBJECT_CLASS (parent_class)->dispose) {
+                G_OBJECT_CLASS (parent_class)->dispose (object);
         }
 }
 
@@ -400,7 +368,7 @@ get_assignment_bounds (MrpAssignment *assign,
 {
         MrpTask *task;
         mrptime  t;
-	
+
         task = mrp_assignment_get_task (assign);
         t = mrp_task_get_work_start (task);
         *x_debut = t * scale;
@@ -440,13 +408,10 @@ static gboolean
 recalc_bounds (PlannerUsageRow *row)
 {
 	PlannerUsageRowPriv *priv;
-	GnomeCanvasItem      *item;
 	gint                  width;
 	gdouble               x_debut, x_fin, x_debut_real;
 	gdouble               old_x, old_x_start, old_width;
 	gboolean              changed;
-	
-	item = GNOME_CANVAS_ITEM (row);
 
 	priv = row->priv;
 
@@ -464,13 +429,13 @@ recalc_bounds (PlannerUsageRow *row)
 	else {
 		width = 0;
 	}
-		
+
 	if (width > 0) {
 		width += TEXT_PADDING;
 	}
 
 	priv->text_width = width;
-    
+
 	if (priv->assignment) {
 		get_assignment_bounds (priv->assignment, priv->scale,
 				       &x_debut, &x_fin, &x_debut_real);
@@ -479,7 +444,7 @@ recalc_bounds (PlannerUsageRow *row)
 		get_resource_bounds (priv->resource, priv->scale, &x_debut,
 				     &x_fin, &x_debut_real);
 	}
-	
+
 	priv->x = x_debut;
 	priv->width = x_fin - x_debut;
 	priv->x_start = x_debut_real;
@@ -568,20 +533,20 @@ usage_row_set_property (GObject      *object,
                                                          "notify",
                                                          G_CALLBACK (usage_row_assignment_notify_cb),
                                                          row, 0);
-				
+
                                 g_signal_connect_object (tmp_task,
                                                          "notify",
                                                          G_CALLBACK (usage_row_task_notify_cb),
                                                          row, 0);
                         }
                 }
-                /*              
+                /*
                  * g_signal_connect_object (priv->resource,
                  * "assignment-added",
                  * G_CALLBACK (usage_row_res_assignment_added),
                  * row,
                  * 0);
-                 * 
+                 *
                  * g_signal_connect_object (priv->resource,
                  * "assignment-removed",
                  * G_CALLBACK (usage_row_res_assignment_removed),
@@ -701,21 +666,20 @@ usage_row_update_resources (PlannerUsageRow *row)
 	PlannerUsageRowPriv *priv;
 	gint                  units;
 	gchar                *units_string;
-    
+
 	priv = row->priv;
-	
+
 	units = mrp_assignment_get_units (priv->assignment);
 	units_string = g_strdup_printf ("%i%%", units);
 	pango_layout_set_text (priv->layout, units_string, -1);
-            
+
 	g_free (units_string);
 }
 
 static void
-usage_row_update (GnomeCanvasItem *item,
-                   double          *affine,
-		   ArtSVP          *clip_path,
-		   gint             flags)
+usage_row_update (GnomeCanvasItem      *item,
+                  const cairo_matrix_t *i2c,
+                  gint                  flags)
 {
         PlannerUsageRow *row;
         double            x1, y1, x2, y2;
@@ -723,8 +687,8 @@ usage_row_update (GnomeCanvasItem *item,
         row = PLANNER_USAGE_ROW (item);
 
         GNOME_CANVAS_ITEM_CLASS (parent_class)->update (item,
-                                                        affine,
-                                                        clip_path, flags);
+                                                        i2c,
+                                                        flags);
 
         usage_row_ensure_layout (row);
         usage_row_get_bounds (row, &x1, &y1, &x2, &y2);
@@ -732,101 +696,35 @@ usage_row_update (GnomeCanvasItem *item,
         gnome_canvas_update_bbox (item, x1, y1, x2, y2);
 }
 
-static void
-usage_row_realize (GnomeCanvasItem * item)
-{
-        PlannerUsageRow     *row;
-        PlannerUsageRowPriv *priv;
+/* TODO-GTK3: Move to GdkRGBA + gdk_cairo_set_source_rgba; later to themeing support */
+#define SET_CAIRO_COLOR(r, g, b) cairo_set_source_rgb (cr, r / 255., g / 255., b / 255.)
 
-        row = PLANNER_USAGE_ROW (item);
-        priv = row->priv;
-
-        GNOME_CANVAS_ITEM_CLASS (parent_class)->realize (item);
-
-        if (complete_stipple == NULL) {
-                complete_stipple = gdk_bitmap_create_from_data (NULL,
-                                                                complete_stipple_pattern,
-                                                                2, 2);
-
-                g_object_add_weak_pointer (G_OBJECT (complete_stipple),
-                                           (gpointer) & complete_stipple);
-
-		gnome_canvas_get_color (item->canvas, "LightSkyBlue3", &color_normal);
-		gnome_canvas_get_color (item->canvas, "#9ac7e0", &color_normal_light);
-		gnome_canvas_get_color (item->canvas, "#7da1b5", &color_normal_dark);
-		
-		gnome_canvas_get_color (item->canvas, "indian red", &color_overuse);
-		gnome_canvas_get_color (item->canvas, "#de6464", &color_overuse_light);
-		gnome_canvas_get_color (item->canvas, "#ba5454", &color_overuse_dark);
-		
-		gnome_canvas_get_color (item->canvas, "grey", &color_underuse);
-		gnome_canvas_get_color (item->canvas, "#d6d6d6", &color_underuse_light);
-		gnome_canvas_get_color (item->canvas, "#a8a8a8", &color_underuse_dark);
-		
-		gnome_canvas_get_color (item->canvas, "medium sea green", &color_free);
-		gnome_canvas_get_color (item->canvas, "#43c77e", &color_free_light);
-		gnome_canvas_get_color (item->canvas, "#359e64", &color_free_dark);
-	} else {
-                g_object_ref (complete_stipple);
-        }
-
-        if (break_stipple == NULL) {
-                break_stipple = gdk_bitmap_create_from_data (NULL,
-                                                             break_stipple_pattern,
-                                                             6, 1);
-
-                g_object_add_weak_pointer (G_OBJECT (break_stipple),
-                                           (gpointer) & break_stipple);
-        } else {
-                g_object_ref (break_stipple);
-        }
-
-        priv->complete_gc = gdk_gc_new (item->canvas->layout.bin_window);
-        gdk_gc_set_stipple (priv->complete_gc, complete_stipple);
-        gdk_gc_set_fill (priv->complete_gc, GDK_STIPPLED);
-
-        priv->break_gc = gdk_gc_new (item->canvas->layout.bin_window);
-        gdk_gc_set_stipple (priv->break_gc, break_stipple);
-        gdk_gc_set_fill (priv->break_gc, GDK_STIPPLED);
-
-        priv->fill_gc = gdk_gc_new (item->canvas->layout.bin_window);
-
-        priv->frame_gc = gdk_gc_new (item->canvas->layout.bin_window);
-        gdk_gc_set_line_attributes (priv->frame_gc,
-                                    0,
-                                    GDK_LINE_SOLID,
-                                    GDK_CAP_BUTT, GDK_JOIN_MITER);
-}
-
-static void
-usage_row_unrealize (GnomeCanvasItem * item)
-{
-        PlannerUsageRow *row;
-
-        row = PLANNER_USAGE_ROW (item);
-
-        gdk_gc_unref (row->priv->complete_gc);
-        row->priv->complete_gc = NULL;
-
-        gdk_gc_unref (row->priv->break_gc);
-        row->priv->break_gc = NULL;
-
-        gdk_gc_unref (row->priv->fill_gc);
-        row->priv->fill_gc = NULL;
-
-	gdk_gc_unref (row->priv->frame_gc);
-        row->priv->frame_gc = NULL;
-
-        if (break_stipple) {
-                g_object_unref (break_stipple);
-        }
-
-        if (complete_stipple) {
-                g_object_unref (complete_stipple);
-        }
-
-        GNOME_CANVAS_ITEM_CLASS (parent_class)->unrealize (item);
-}
+/* LightSkyBlue3 */
+#define SET_COLOR_NORMAL         SET_CAIRO_COLOR(141, 182, 205)
+/* #9ac7e0 */
+#define SET_COLOR_NORMAL_LIGHT   SET_CAIRO_COLOR(0x9a, 0xc7, 0xe0)
+/* #7da1b5 */
+#define SET_COLOR_NORMAL_DARK    SET_CAIRO_COLOR(0x7d, 0xa1, 0xb5)
+/* indian red */
+#define SET_COLOR_OVERUSE        SET_CAIRO_COLOR(205, 92, 92)
+/* #de6464 */
+#define SET_COLOR_OVERUSE_LIGHT  SET_CAIRO_COLOR(0xde, 0x64, 0x64)
+/* #ba5454 */
+#define SET_COLOR_OVERUSE_DARK   SET_CAIRO_COLOR(0xba, 0x54, 0x54)
+/* grey */
+#define SET_COLOR_UNDERUSE       SET_CAIRO_COLOR(190, 190, 190)
+/* #d6d6d6 */
+#define SET_COLOR_UNDERUSE_LIGHT SET_CAIRO_COLOR(0xd6, 0xd6, 0xd6)
+/* #a8a8a8 */
+#define SET_COLOR_UNDERUSE_DARK  SET_CAIRO_COLOR(0xa8, 0xa8, 0xa8)
+/* medium sea green */
+#define SET_COLOR_FREE           SET_CAIRO_COLOR(60, 179, 113)
+/* #43c77e */
+#define SET_COLOR_FREE_LIGHT     SET_CAIRO_COLOR(0x43, 0xc7, 0x7e)
+/* #359e64 */
+#define SET_COLOR_FREE_DARK      SET_CAIRO_COLOR(0x35, 0x9e, 0x64)
+/* LightSkyBlue1 */
+#define SET_COLOR_COMPLETE       SET_CAIRO_COLOR(176, 226, 255)
 
 typedef enum {
         START_ASSIGN,
@@ -879,7 +777,7 @@ usage_row_draw_resource_ival (mrptime          start,
                                mrptime          end,
                                gint             units,
                                RowChunk         chunk,
-                               GdkDrawable     *drawable,
+                               cairo_t         *cr,
                                GnomeCanvasItem *item,
                                gint             x,
 			       gint             y,
@@ -950,7 +848,7 @@ usage_row_draw_resource_ival (mrptime          start,
         cr_ystart = cs_ystart + 1;
         cr_xend = c_xend;
         cr_yend = cs_yend - 1;
-	
+
         /* Clip to the expose area */
 	r_xstart = MAX (c_xstart, 0);
         r_xend = MIN (c_xend, width);
@@ -972,119 +870,129 @@ usage_row_draw_resource_ival (mrptime          start,
                 return;
         }
 
+        cairo_save (cr);
+        cairo_set_line_width (cr, 1.0);
+        cairo_set_line_cap (cr, CAIRO_LINE_CAP_SQUARE);
+
         if (units == 0) {
-                gdk_gc_set_foreground (priv->fill_gc, &color_free);
+                SET_COLOR_FREE;
         }
 	else if (units < 100) {
-                gdk_gc_set_foreground (priv->fill_gc, &color_underuse);
+                SET_COLOR_UNDERUSE;
         }
 	else if (units == 100) {
-                gdk_gc_set_foreground (priv->fill_gc, &color_normal);
+                SET_COLOR_NORMAL;
         } else {
-                gdk_gc_set_foreground (priv->fill_gc, &color_overuse);
+                SET_COLOR_OVERUSE;
         }
 
         /* Draw the central part of the chunk */
 	if (rr_xend >= rr_xstart && rr_yend >= rr_ystart) {
-		gdk_draw_rectangle (drawable,
-				    priv->fill_gc,
-				    TRUE,
-				    rr_xstart, rr_ystart,
-				    rr_xend - rr_xstart + 1, rr_yend - rr_ystart + 1);
+		cairo_rectangle (cr,
+				 rr_xstart, rr_ystart,
+				 rr_xend - rr_xstart + 1, rr_yend - rr_ystart + 1);
+		cairo_fill (cr);
 	}
 
         if (units == 0) {
-                gdk_gc_set_foreground (priv->fill_gc, &color_free_light);
+                SET_COLOR_FREE_LIGHT;
         }
 	else if (units < 100) {
-                gdk_gc_set_foreground (priv->fill_gc, &color_underuse_light);
+                SET_COLOR_UNDERUSE_LIGHT;
         }
 	else if (units == 100) {
-                gdk_gc_set_foreground (priv->fill_gc, &color_normal_light);
+                SET_COLOR_NORMAL_LIGHT;
         } else {
-                gdk_gc_set_foreground (priv->fill_gc, &color_overuse_light);
+                SET_COLOR_OVERUSE_LIGHT;
         }
-	
-        //gdk_gc_set_foreground (priv->fill_gc, &color_high);
 
+        /* TODO: Simplify drawing with cairo_rel_line_to or cairo_rectangle? */
         /* Top of the shadow. */
         if (cs_ystart == rs_ystart) {
-                gdk_draw_line (drawable, priv->fill_gc, r_xstart, rs_ystart,
-                               r_xend, rs_ystart);
+                cairo_move_to (cr, r_xstart + 0.5, rs_ystart + 0.5);
+                cairo_line_to (cr, r_xend + 0.5, rs_ystart + 0.5);
         }
-	
+
         /* Left of the shadow. */
         if (chunk & ROW_START && cs_xstart == rs_xstart) {
-                gdk_draw_line (drawable, priv->fill_gc, rs_xstart, rs_ystart,
-                               rs_xstart, cs_yend);
+                cairo_move_to (cr, rs_xstart + 0.5, rs_ystart + 0.5);
+                cairo_line_to (cr, rs_xstart + 0.5, cs_yend + 0.5);
         }
-	
+
+        cairo_stroke (cr);
+
 	if (units == 0) {
-                gdk_gc_set_foreground (priv->fill_gc, &color_free_dark);
+                SET_COLOR_FREE_DARK;
         }
 	else if (units < 100) {
-                gdk_gc_set_foreground (priv->fill_gc, &color_underuse_dark);
+                SET_COLOR_UNDERUSE_DARK;
         }
 	else if (units == 100) {
-                gdk_gc_set_foreground (priv->fill_gc, &color_normal_dark);
+                SET_COLOR_NORMAL_DARK;
         } else {
-                gdk_gc_set_foreground (priv->fill_gc, &color_overuse_dark);
+                SET_COLOR_OVERUSE_DARK;
         }
-	
-	//gdk_gc_set_foreground (priv->fill_gc, &color_shadow);
 
         /* Bottom of the shadow. */
         if (cs_yend == rs_yend) {
-                gdk_draw_line (drawable, priv->fill_gc, r_xstart, rs_yend,
-                               r_xend, rs_yend);
+                cairo_move_to (cr, r_xstart + 0.5, rs_yend + 0.5);
+                cairo_line_to (cr, r_xend + 0.5, rs_yend + 0.5);
         }
-	
+
         /* Right of the shadow. */
         if (chunk & ROW_END && cs_xend == rs_xend) {
-                gdk_draw_line (drawable, priv->fill_gc, rs_xend, rs_ystart,
-                               rs_xend, cs_yend);
+                cairo_move_to (cr, rs_xend + 0.5, rs_ystart + 0.5);
+                cairo_line_to (cr, rs_xend + 0.5, cs_yend + 0.5);
         }
 
+        cairo_stroke (cr);
+
 	/* Interval separator. */
 	if (!(chunk & ROW_START)) {
-		gdk_gc_set_foreground (priv->fill_gc, &GTK_WIDGET (item->canvas)->style->white);
-		gdk_draw_line (drawable, priv->fill_gc, c_xstart, rs_ystart, c_xstart,
-                               rr_yend);
+		cairo_set_source_rgb (cr, 1.0, 1.0, 1.0); /* white */
+		cairo_move_to (cr, c_xstart + 0.5, rs_ystart + 0.5);
+		cairo_line_to (cr, c_xstart + 0.5, rr_yend + 0.5);
+		cairo_stroke (cr);
 	}
-	
+
+        cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+
         /* Top frame. */
         if (c_ystart == r_ystart) {
-                gdk_draw_line (drawable, priv->frame_gc, r_xstart, r_ystart, r_xend,
-                               r_ystart);
+                cairo_move_to (cr, r_xstart + 0.5, r_ystart + 0.5);
+                cairo_line_to (cr, r_xend + 0.5, r_ystart + 0.5);
 	}
-	
+
         /* Bottom frame. */
         if (c_yend == r_yend) {
-                gdk_draw_line (drawable, priv->frame_gc, r_xstart, r_yend, r_xend,
-                               r_yend);
+                cairo_move_to (cr, r_xstart + 0.5, r_yend + 0.5);
+                cairo_line_to (cr, r_xend + 0.5, r_yend + 0.5);
 	}
-	
+
         /* Left frame. */
         if (chunk & ROW_START && c_xstart == r_xstart) {
-                gdk_draw_line (drawable, priv->frame_gc, r_xstart, r_ystart,
-                               r_xstart, r_yend);
+                cairo_move_to (cr, r_xstart + 0.5, r_ystart + 0.5);
+                cairo_line_to (cr, r_xstart + 0.5, r_yend + 0.5);
 	}
-	
+
         /* Right frame. */
         if (chunk & ROW_END && c_xend == r_xend) {
-                gdk_draw_line (drawable, priv->frame_gc, r_xend, r_ystart, r_xend,
-                               r_yend);
+                cairo_move_to (cr, r_xend + 0.5, r_ystart + 0.5);
+                cairo_line_to (cr, r_xend + 0.5, r_yend + 0.5);
 	}
+
+        cairo_stroke (cr);
+        cairo_restore (cr);
 }
 
 static void
 usage_row_draw_resource (PlannerUsageRow *row,
-                          GdkDrawable      *drawable,
-                          GnomeCanvasItem  *item,
-                          gint              x,
-			  gint              y,
-			  gint              width,
-			  gint              height)
+                         cairo_t         *cr,
+                         GnomeCanvasItem *item,
+                         gint             x,
+                         gint             y,
+                         gint             width,
+                         gint             height)
 {
         GList         *dates;
         MrpResource   *resource;
@@ -1152,7 +1060,7 @@ usage_row_draw_resource (PlannerUsageRow *row,
                                                        date->time,
                                                        units,
                                                        chunk,
-                                                       drawable, item,
+                                                       cr, item,
                                                        x, y, width, height);
 
                         chunk &= ~ROW_START;
@@ -1169,12 +1077,12 @@ usage_row_draw_resource (PlannerUsageRow *row,
         g_list_free (dates);
 
         if (!(chunk & ROW_END)) {
-		chunk |= ROW_END;	
+		chunk |= ROW_END;
                 usage_row_draw_resource_ival (previous_time,
                                                finish,
                                                units,
                                                chunk,
-                                               drawable, item,
+                                               cr, item,
                                                x, y, width, height);
         }
 }
@@ -1183,13 +1091,13 @@ static void
 usage_row_draw_assignment (PlannerUsageRow *row,
                             MrpAssignment    *assign,
                             GnomeCanvasItem  *item,
-                            GdkDrawable      *drawable,
+                            cairo_t          *cr,
                             gint              x,
 			    gint              y,
 			    gint              width,
 			    gint              height)
 {
-        PlannerUsageRowPriv *priv;
+        PlannerUsageRowPriv  *priv;
         MrpTask              *task;
         gdouble               i2w_dx;
         gdouble               i2w_dy;
@@ -1197,8 +1105,8 @@ usage_row_draw_assignment (PlannerUsageRow *row,
         gboolean              summary;
         gint                  percent_complete;
         gint                  complete_x2, complete_width;
-        gint                  rx1, ry1;
-        gint                  rx2, ry2;
+        gint                  rx1;
+        gint                  rx2;
         gint                  cx1, cy1, cx2, cy2;
         gdouble               ass_x, ass_xend, ass_x_start;
 
@@ -1231,101 +1139,102 @@ usage_row_draw_assignment (PlannerUsageRow *row,
                 return;
         }
 
+        cairo_save (cr);
+        cairo_set_line_width (cr, 1.0);
+        cairo_set_line_cap (cr, CAIRO_LINE_CAP_SQUARE);
+
         /* summary_y = floor (priv->y + 2 * 0.15 * priv->height + 0.5) - y; */
 
         /* "Clip" the expose area. */
         rx1 = MAX (cx1, 0);
         rx2 = MIN (cx2, width);
 
-        ry1 = MAX (cy1, 0);
-        ry2 = MIN (cy2, height);
-
         summary = (mrp_task_get_n_children (task) > 0);
         complete_width = 0;
         complete_x2 = 0;
 
         if (!summary) {
                 percent_complete = mrp_task_get_percent_complete (task);
-		
+
 		complete_width =  floor ((cx2 - cx1) * (percent_complete / 100.0) + 0.5);
                 complete_x2 = MIN (cx1 + complete_width, rx2);
         }
-	
+
         if (rx1 <= rx2) {
-                if (complete_width > 0) {
-                        gnome_canvas_set_stipple_origin (item->canvas,
-                                                         priv->complete_gc);
-                }
-
-                gdk_gc_set_foreground (priv->fill_gc, &color_normal);
-
-                gdk_draw_rectangle (drawable,
-                                    priv->fill_gc,
-                                    TRUE,
-                                    rx1, cy1 + 1, rx2 - rx1, cy2 - cy1 - 1);
+                SET_COLOR_NORMAL;
+                cairo_rectangle (cr, rx1, cy1 + 1, rx2 - rx1, cy2 - cy1 - 1);
+                cairo_fill (cr);
 
                 if (rx1 <= complete_x2) {
-                        gdk_draw_rectangle (drawable,
-                                            priv->complete_gc,
-                                            TRUE,
-                                            rx1,
-                                            cy1 + 4,
-                                            complete_x2 - rx1, cy2 - cy1 - 7);
+                        /* TODO: Improve design of completed percentage bar; perhaps add borders? */
+                        SET_COLOR_COMPLETE;
+                        cairo_rectangle (cr,
+                                         rx1, cy1 + 5,
+                                         complete_x2 - rx1, cy2 - cy1 - 9);
+                        cairo_fill (cr);
                 }
 
-                gdk_draw_line (drawable, priv->frame_gc, rx1, cy1, rx2, cy1);
-                gdk_draw_line (drawable, priv->frame_gc, rx1, cy2, rx2, cy2);
+                cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+                cairo_move_to (cr, rx1 + 0.5, cy1 + 0.5);
+                cairo_line_to (cr, rx2 + 0.5, cy1 + 0.5);
+                cairo_move_to (cr, rx1 + 0.5, cy2 + 0.5);
+                cairo_line_to (cr, rx2 + 0.5, cy2 + 0.5);
+                cairo_stroke (cr);
 
-                gdk_gc_set_foreground (priv->fill_gc, &color_normal_light);
-                gdk_draw_line (drawable,
-                               priv->fill_gc,
-                               rx1 + 0, cy1 + 1, rx2 - 0, cy1 + 1);
+                SET_COLOR_NORMAL_LIGHT;
+                cairo_move_to (cr, rx1 + 0 + 0.5, cy1 + 1 + 0.5);
+                cairo_line_to (cr, rx2 - 0 + 0.5, cy1 + 1 + 0.5);
 
                 if (cx1 == rx1) {
-                        gdk_draw_line (drawable,
-                                       priv->fill_gc,
-                                       rx1 + 1, cy1 + 1, rx1 + 1, cy2 - 1);
+                        cairo_move_to (cr, rx1 + 1 + 0.5, cy1 + 1 + 0.5);
+                        cairo_line_to (cr, rx1 + 1 + 0.5, cy2 - 1 + 0.5);
                 }
 
-                gdk_gc_set_foreground (priv->fill_gc, &color_normal_dark);
-                gdk_draw_line (drawable,
-                               priv->fill_gc,
-                               rx1 + 0, cy2 - 1, rx2 - 0, cy2 - 1);
+                cairo_stroke (cr);
+
+                SET_COLOR_NORMAL_DARK;
+                cairo_move_to (cr, rx1 + 0 + 0.5, cy2 - 1 + 0.5);
+                cairo_line_to (cr, rx2 - 0 + 0.5, cy2 - 1 + 0.5);
 
                 if (cx2 == rx2) {
-                        gdk_draw_line (drawable,
-                                       priv->fill_gc,
-                                       rx2 - 1, cy1 + 1, rx2 - 1, cy2 - 1);
+                        cairo_move_to (cr, rx2 - 1 + 0.5, cy1 + 1 + 0.5);
+                        cairo_line_to (cr, rx2 - 1 + 0.5, cy2 - 1 + 0.5);
                 }
 
+                cairo_stroke (cr);
+
+                cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
+
                 if (cx1 == rx1) {
-                        gdk_draw_line (drawable,
-                                       priv->frame_gc, rx1, cy1, rx1, cy2);
+                        cairo_move_to (cr, rx1 + 0.5, cy1 + 0.5);
+                        cairo_line_to (cr, rx1 + 0.5, cy2 + 0.5);
                 }
 
                 if (cx2 == rx2) {
-                        gdk_draw_line (drawable,
-                                       priv->frame_gc, rx2, cy1, rx2, cy2);
+                        cairo_move_to (cr, rx2 + 0.5, cy1 + 0.5);
+                        cairo_line_to (cr, rx2 + 0.5, cy2 + 0.5);
                 }
+
+                cairo_stroke (cr);
         }
-        
+
         rx1 = MAX (cx2 + TEXT_PADDING, 0);
         rx2 = MIN (cx2 + TEXT_PADDING + priv->text_width, width);
 
-        if (priv->layout != NULL && rx1 < rx2) {
-		/* FIXME: Center the text vertically? */
-		gdk_draw_layout (drawable,
-				 GTK_WIDGET (item->canvas)->style->text_gc[GTK_STATE_NORMAL],
-				 cx2 + TEXT_PADDING,
-				 cy1,
-				 priv->layout);
+	if (priv->layout != NULL && rx1 < rx2) {
+		cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
 
-        }
+		/* FIXME: Center the text vertically? */
+		cairo_move_to (cr, cx2 + TEXT_PADDING, cy1);
+		pango_cairo_show_layout (cr, priv->layout);
+	}
+
+        cairo_restore (cr);
 }
 
 static void
 usage_row_draw (GnomeCanvasItem *item,
-                 GdkDrawable     *drawable,
+                 cairo_t        *cr,
                  gint             x,
 		 gint             y,
 		 gint             width,
@@ -1339,32 +1248,28 @@ usage_row_draw (GnomeCanvasItem *item,
                 usage_row_draw_assignment (row,
                                             row->priv->assignment,
                                             item,
-                                            drawable, x, y, width, height);
+                                            cr, x, y, width, height);
         }
 	else if (row->priv->resource) {
-                usage_row_draw_resource (row, drawable, item, x, y, width, height);
+                usage_row_draw_resource (row, cr, item, x, y, width, height);
         }
 }
 
-static double
+static GnomeCanvasItem *
 usage_row_point (GnomeCanvasItem  *item,
                   double            x,
                   double            y,
 		  gint              cx,
-		  gint              cy,
-		  GnomeCanvasItem **actual_item)
+		  gint              cy)
 {
         PlannerUsageRow     *row;
         PlannerUsageRowPriv *priv;
         gint                  text_width;
         gdouble               x1, y1, x2, y2;
-        gdouble               dx, dy;
 
         row = PLANNER_USAGE_ROW (item);
         priv = row->priv;
 
-        *actual_item = item;
-
         text_width = priv->text_width;
         if (text_width > 0) {
 		text_width += TEXT_PADDING;
@@ -1376,29 +1281,11 @@ usage_row_point (GnomeCanvasItem  *item,
         y2 = y1 + priv->height;
 
         if (x > x1 && x < x2 && y > y1 && y < y2) {
-                return 0.0;
+                return item;
         }
 
         /* Point is outside rectangle */
-        if (x < x1) {
-                dx = x1 - x;
-        }
-	else if (x > x2) {
-                dx = x - x2;
-        } else {
-                dx = 0.0;
-        }
-
-        if (y < y1) {
-                dy = y1 - y;
-        }
-	else if (y > y2) {
-                dy = y - y2;
-        } else {
-                dy = 0.0;
-        }
-
-        return sqrt (dx * dx + dy * dy);
+        return NULL;
 }
 
 static void
@@ -1464,7 +1351,7 @@ usage_row_assignment_notify_cb (MrpAssignment    *assignment,
         if (!recalc_bounds (row)) {
 		return;
 	}
-	
+
         usage_row_geometry_changed (row);
         gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (row));
 }
@@ -1487,7 +1374,7 @@ usage_row_task_notify_cb (MrpTask          *task,
         if (!recalc_bounds (row)) {
 		return;
 	}
-	
+
         usage_row_geometry_changed (row);
         gnome_canvas_item_request_update (GNOME_CANVAS_ITEM (row));
 }
@@ -1551,7 +1438,7 @@ usage_row_geometry_changed (PlannerUsageRow * row)
         if (row->priv->assignment) {
 		usage_row_update_resources (row);
         }
-        
+
         x1 = row->priv->x;
         y1 = row->priv->y;
         x2 = x1 + row->priv->width;
@@ -1571,14 +1458,12 @@ usage_row_event (GnomeCanvasItem *item, GdkEvent *event)
 	PlannerUsageTree    *tree;
 	GtkTreeView         *tree_view;
 	GtkTreeIter          iter;
-	GtkWidget           *canvas_widget;
 
 	row = PLANNER_USAGE_ROW (item);
 	priv = row->priv;
-	canvas_widget = GTK_WIDGET (item->canvas);
 	chart = g_object_get_data (G_OBJECT (item->canvas), "chart");
 	tree = planner_usage_chart_get_view (chart);
-	
+
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
 		if (priv->assignment != NULL) {
@@ -1597,10 +1482,10 @@ usage_row_event (GnomeCanvasItem *item, GdkEvent *event)
 
 		tree_view = GTK_TREE_VIEW (tree);
 		selection = gtk_tree_view_get_selection (tree_view);
-		
-		gtk_tree_model_get_iter (gtk_tree_view_get_model (tree_view), 
+
+		gtk_tree_model_get_iter (gtk_tree_view_get_model (tree_view),
 					 &iter, path);
-		
+
 		if (!gtk_tree_selection_iter_is_selected (selection, &iter)) {
 			gtk_tree_selection_unselect_all (selection);
 			gtk_tree_selection_select_path (selection, path);
@@ -1615,16 +1500,16 @@ usage_row_event (GnomeCanvasItem *item, GdkEvent *event)
 			}
 			else if (priv->resource != NULL) {
 				planner_usage_tree_edit_resource (tree);
-			}	
+			}
 		}
 
 		break;
-		
+
 	default:
 		break;
 	}
-	
-			
+
+
 
 	if (TRUE) {
 		return TRUE;
